
## Instructions

Learning Objective
- By the end of this tutorial, you can have an app with multiple screens. Clicking a button or text will open up another screen. And you can setup some screens to allow users to swipe away in order to go back a screen.


Create a new expo:
```
npx create-expo APP --example navigation
```

That also included react-navigation’s bottom tabs which you may want to remove in the code, but it HAS the Stack Navigator. We want to remove the tabs for now so we can just focus on the Stack Navigator. On a future challenge, we can focus on learning the tabs which can be combined with stack navigator.

Remove all of everything inside App() and its imports besides safe areas:
```
import "react-native-gesture-handler";  
  
import { StatusBar } from "expo-status-bar";  
import { SafeAreaProvider } from "react-native-safe-area-context";  
  
import { useLoadedAssets } from "./hooks/useLoadedAssets";  
import { useColorScheme } from "react-native";  
  
export default function App() {  
  const isLoadingComplete = useLoadedAssets();  
  const colorScheme = useColorScheme();  
  
  if (!isLoadingComplete) {  
    return null;  
  } else {  
    return (  
      <SafeAreaProvider>  
        <Navigation colorScheme={colorScheme} />  
        <StatusBar />  
      </SafeAreaProvider>  
    );  
  }  
}
```

-->

```
import "react-native-gesture-handler";  
import { SafeAreaProvider } from "react-native-safe-area-context";  
import { View,Text,TouchableOpacity } from 'react-native';  
  
export default function App() {  
    return (  
      <SafeAreaProvider>  
        <View>  
          <Text>Test</Text>  
        </View>  
      </SafeAreaProvider>  
    );  
}
```

The `<NavigationContainer>` component should wrap **all** of your navigators, not just one. It acts as the root component for your navigation structure, providing the navigation context that all navigators within your app share.
```
import "react-native-gesture-handler";  
import { SafeAreaProvider } from "react-native-safe-area-context";  
import { View,Text,TouchableOpacity } from 'react-native';  
import { NavigationContainer } from '@react-navigation/native';  
  
export default function App() {  
    return (  
      <SafeAreaProvider>  
          <NavigationContainer>  
            {/* Rest of your app code */}  
          </NavigationContainer>  
      </SafeAreaProvider>  
    );  
}
```

When an user is at a screen that has a Stack Navigator, which configures all possible screens that can be linked in a stack of swipable screens, the first screen in that configuration is what immediately renders.

It doesn’t matter that your screens follow in the same direction as they are configured in the Navigator. You just need pressibles or touchables that run navigate method to the string of the screen name when pressed or interacted with. As long as the next screen is within in the Navigator configurations, user can swipe back to the previous screen or hit back on the previous screen and then the user sees an animation of  the current screen getting rid of and the previous screen showing underneath.

The Stack Navigator follows a builder pattern of getting a component then using the component to build out the stack. That group of components must be some descendant of `<NavigationContainer>`  which follows a React provider pattern (A structured way of managing and passing data through your component tree; In this case, it was needed because the app needs to handle the possibility of multiple Stack Navigators or nested Stack Navigators).

1. You create a stack navigator and get a configurable component in return. Make sure to run this globally outside of your component. Here are additional imports and lines of code:

```
import { createStackNavigator } from "@react-navigation/stack";  
import TabOneScreen from './screens/TabOneScreen'  
import TabTwoScreen from './screens/TabTwoScreen'  
  
const Stack = createStackNavigator();
```

2. Notice to configure, you use that component’s subcomponents: Navigator>(Screen\*N)
```
export default function App() {  
 return (  
    {/* ... */}  
    <Stack.Navigator screenOptions={{ headerShown: true }}>  
      <Stack.Screen name="TabOne" component={TabOneScreen} /> {/* Renders automatically for this file */}  
      <Stack.Screen name="TabTwo" component={TabTwoScreen} />  
    </Stack.Navigator>  
    {/* ... */}  
)
```


Btw, you are not limited to calling the configurable component “Stack”. It was just for a lack of words here. You could have:
> ```
> const Screens = createStackNavigator();
> 
> export default function App() {
> return (
>   {/* ... */}
>      <Screens.Navigator screenOptions={{ headerShown: false }}>
>        <Screens.Screen name="TabOne" component={TabOneScreen} />
 >      <Screens.Screen name="TabTwo" component={TabTwoScreen} />
>      </Screens.Navigator>
   {/* ... */}
> )
> ```

Laying out those subcomponents will tell the app when you run `navigation.navigate(”TabTwo”)`  (details after these explanations), it will open the TabTwo page. Then user can swipe it away to review the previous page (which is TabOne where you’re at). You can have more than 2 configured screens and you are not required to navigate them in the same order as the stack configuration listed them.  To prove the point, for example, you might navigate to the "NotFound" screen by calling `navigation.navigate('NotFound')` within your app  at any screen that’s part of the same list at .Navigator>(.Stack\*N)

Technically, when you call navigation.navigate, React Navigation determines which navigator you're currently using and tries to find the screen within the available navigators. If a screen is found, it'll either push to the stack or unwind until you're on that page.

Navigate from a current page to another page that exists in the configured Navigator list:
```
import { useNavigation } from "@react-navigation/native";  
  
... onPress={()=> {  
 navigation.navigate("TabTwo");  }  
}) ...
```

As long as "TabTwo" was configured with a component in the _Navigator>_ .Screen+.Screen

You can create pressibles, touchable’s, or buttons that can take onPress. You will mimic these screenshots. You will have to create an additional screen (named route "TabTwoDetails”, component "TabTwoDetailsScreen”, file “screens/TabTwoDetailsScreen”) and add it to the Stack Navigator list.

----

## Instructions - Mimic

Mimic the screens and stack of screens (Stack Navigator):

**When open app**
![](eoIqhCt.png)


b: When clicked "Screen Two":
![](U0GXvvw.png)

c: When clicked "Continue to Screen Two Details"
![](F2jGYlL.png)

Then hitting the back at the top takes you back to “b” and you saw a swiping animation if done on phone

Next, compare your work with the answer section next or skip to the last section "Discussions"

---

## Answer


**Given up or comparing after you finished?**

App.js:
```
import "react-native-gesture-handler";  
  
import {NavigationContainer} from "@react-navigation/native";  
import {createStackNavigator} from "@react-navigation/stack";  
  
import { SafeAreaView, View} from "react-native";  
import { SafeAreaProvider } from "react-native-safe-area-context";  
  
import TabOneScreen from './screens/TabOneScreen'  
import TabTwoScreen from './screens/TabTwoScreen'  
import TabTwoDetailsScreen from './screens/TabTwoDetailsScreen'  
  
const Stack = createStackNavigator();  
  
export default function App() {  
  return (  
    <SafeAreaProvider>  
    <View style={{flex:1, gap:"10px", height:"100%", width:"100%"}}>  
      <NavigationContainer>  
        <Stack.Navigator screenOptions={{ headerShown: true }}>  
          <Stack.Screen name="TabOne" component={TabOneScreen}/>  
          <Stack.Screen name="TabTwo" component={TabTwoScreen}/>  
          <Stack.Screen name="TabTwoDetails" component={TabTwoDetailsScreen} />  
        </Stack.Navigator>  
      </NavigationContainer>  
    </View>  
    </SafeAreaProvider>  
  );  
}
```

screens/TabOneScreen.js:
```
import { StyleSheet, Text, View, Button, TouchableOpacity } from "react-native";  
import {useNavigation} from "@react-navigation/native";  
  
export default function ScreenOne() {  
  const navigation = useNavigation();  
  return (  
    <View style={styles.container}>  
        <Button title="Screen One" onPress={()=>{ navigation.navigate("TabOne") }} style={styles.gap}></Button>  
        <VertSpace></VertSpace>  
        <Button title="Screen Two" onPress={()=>{ navigation.navigate("TabTwo") }} style={styles.gap}></Button>  
    </View>  
  );  
}  
  
const VertSpace = () => {
  return (<View style={{height:"20px", width:"1px"}}><Text></Text></View>)
}
  
const styles = StyleSheet.create({  
  container: {  
    flex: 1,  
    alignItems: "center",  
    justifyContent: "center",  
  },  
  title: {  
    fontSize: 20,  
    fontWeight: "bold",  
  },  
  gap: {  
    marginVertical: 10  
  }  
});
```

screens/TabTwoScreen.js:
Notice .navigate at “Continue to Screen Two Details”
```
import { StyleSheet, Text, View, Button, TextInput, TouchableOpacity } from "react-native";  
import {useNavigation} from "@react-navigation/native";  
// import {createStackNavigator} from "@react-navigation/native";  
import {useState} from 'react'  
  
export default function ScreenTwo() {  
  const [firstName, setFirstName] = useState("")  
  const [lastName, setLastName] = useState("")  
  
  const navigation = useNavigation();  
  return (  
    <View style={styles.container}>  
      <Button title="Screen One" onPress={()=>{ navigation.navigate("TabOne") }} style={styles.gap}></Button>  
      <VertSpace></VertSpace>  
      <Button title="Screen Two" onPress={()=>{ navigation.navigate("TabTwo") }} style={styles.gap}></Button>  
      <View  
        style={styles.separator}  
        lightColor="#eee"  
        darkColor="rgba(255,255,255,0.1)"  
      />  
      <TouchableOpacity onPress={()=>{   
        navigation.navigate("TabTwoDetails")  
      }} style={styles.gap}>  
        <Text>Continue to Screen Two Details</Text>  
      </TouchableOpacity>  
    </View>  
  );  
}  
  
const VertSpace = () => {
  return (<View style={{height:"20px", width:"1px"}}><Text></Text></View>)
}
  
const styles = StyleSheet.create({  
  container: {  
    flex: 1,  
    alignItems: "center",  
    justifyContent: "center",  
  },  
  title: {  
    fontSize: 20,  
    fontWeight: "bold",  
  },  
  separator: {  
    marginVertical: 30,  
    height: 1,  
    width: "80%",  
  },  
  gap: {  
    marginVertical: 10  
  }  
});
```

screens/TabTwoDetailsScreen.js:
```
import { StyleSheet, Text, View } from "react-native"  
  
export default function ScreenTwoDetails(  
) {  
  return (  
    <View style={styles.container}>  
      <Text style={styles.title}>Screen Two Detail</Text>  
      <View  
        style={styles.separator}  
        lightColor="#eee"  
        darkColor="rgba(255,255,255,0.1)"  
      />  
    </View>  
  );  
}  
  
const VertSpace = () => {
  return (<View style={{height:"20px", width:"1px"}}><Text></Text></View>)
} 
  
const styles = StyleSheet.create({  
  container: {  
    flex: 1,  
    alignItems: "center",  
    justifyContent: "center",  
  },  
  title: {  
    fontSize: 20,  
    fontWeight: "bold",  
  },  
  separator: {  
    marginVertical: 30,  
    height: 1,  
    width: "80%",  
  },  
});
```

---

## Discussions

**Further discussion - Named Routes**

Notice when you run .navigate, you pass it the string from a defined screen in Stack Navigator. This identifiable route that you tell the app to navigate to, that’s a named route. In other implementations you may pass to .navigate routes that start with a /, like /tab-one - that’s a url route. The url route could be react-router-native or it could be based on the file structure which is expo-router inspired by Next JS’ routing system.

**Further discussion - Nested Stack Navigators**

Keep in mind more advance use which is often used in many situations: You can nest navigators  to create hierarchies of screens. For another Stack Navigator, you do the same of creating a Stack Navigator (`<NavigationController>` ) and then configuring a bunch of screens that can be navigated between and historically tracked.; But you probably want to name that component you get from `createStackNavigator()` differently than the other components you get from the Stack Navigators.

Mini-Challenge: Create another Stack Navigator for "Screen One" that goes into "Screen One Details". It is separate from "Screen Two" and "Screen Two Details". Test swiping to see it's historically tracked.